{-# LANGUAGE DeriveGeneric #-}

module Messages (
  Message (..),
  TimestampedMessage (..),
  currentTimeMillis,
  timestamped,
  ServerMessage (..)
) where

import GHC.Generics
import Data.Aeson
import Data.Aeson.Types
import Data.Time.Clock.POSIX (getPOSIXTime)
import Control.Concurrent.MVar
import Hakka.Actor (ActorRef)

data Message = Ping | Pong deriving (Generic,Show)

currentTimeMillis :: IO Integer
currentTimeMillis = (round . (1000 *)) `fmap` getPOSIXTime

timestamped :: Message -> IO TimestampedMessage
timestamped msg = currentTimeMillis >>= \time -> return $ TimestampedMessage time msg

data TimestampedMessage = TimestampedMessage {
  timestamp :: Integer,
  payload :: Message
} deriving (Generic,Show)

data ServerMessage = 
    Connect {
      socketOut :: (TimestampedMessage -> IO ()),
      connectionActor :: MVar (ActorRef ServerMessage)
    }
  | Disconnect
  | Msg TimestampedMessage
  | Ok

instance Show ServerMessage where
  show (Connect _ _)  = "Connect <...>"
  show Disconnect = "Disconnect"
  show (Msg x) = "Msg " ++ show x
  show Ok = "Ok"

----------------------
-- JSON En/Decoding --

jsonOptions :: Options
jsonOptions = defaultOptions {
  sumEncoding = ObjectWithSingleField,
  allNullaryToStringTag = False
}

instance ToJSON Message where
  toEncoding = genericToEncoding jsonOptions

instance FromJSON Message where
  parseJSON = genericParseJSON jsonOptions

instance ToJSON TimestampedMessage where
  toEncoding = genericToEncoding jsonOptions

instance FromJSON TimestampedMessage where
  parseJSON = genericParseJSON jsonOptions
