import io.circe._
import io.circe.generic.semiauto._

case class TimestampedMessage(
  timestamp: Long,
  payload: Message
)

sealed trait Message
case object Ping extends Message
case object Pong extends Message

///////////////////////////
// JSON De/serialization //
///////////////////////////

sealed trait JSONConfig {
  import io.circe.Encoder
  import shapeless.{ Generic, HNil }

  implicit def encodeCaseObject[A <: Product](implicit
    gen: Generic.Aux[A, HNil]
  ): Encoder[A] = Encoder.instance(_ => Json.arr())
}

object TimestampedMessage extends JSONConfig {
  def apply(payload: Message): TimestampedMessage = TimestampedMessage(System.currentTimeMillis(),payload)

  implicit val encodeTimestampedMessage: Encoder[TimestampedMessage] =
    deriveEncoder[TimestampedMessage]

  implicit val decodeTimestampedMessage: Decoder[TimestampedMessage] =
    deriveDecoder[TimestampedMessage]
}

object Message extends JSONConfig {
  implicit val encodeMessage: Encoder[Message] =
    deriveEncoder[Message]

  implicit val decodeMessage: Decoder[Message] =
    deriveDecoder[Message]
}